@@ -16,13 +16,14 @@ if gevent_socketio_found:
 import socketio
 import flask
 from flask import _request_ctx_stack, json as flask_json
+from flask.sessions import SessionMixin
 from werkzeug.debug import DebuggedApplication
 from werkzeug.serving import run_with_reloader
 
 from .namespace import Namespace
 from .test_client import SocketIOTestClient
 
-__version__ = '2.9.1'
+__version__ = '2.9.4'
 
 
 class _SocketIOMiddleware(socketio.Middleware):
@@ -42,6 +43,13 @@ class _SocketIOMiddleware(socketio.Middleware):
                                                          start_response)
 
 
+class _ManagedSession(SessionMixin, dict):
+    """This class is used for user sessions that are managed by
+    Flask-SocketIO. It is simple dict, expanded with the Flask session
+    attributes."""
+    pass
+
+
 class SocketIO(object):
     """Create a Flask-SocketIO server.
 
@@ -81,7 +89,8 @@ class SocketIO(object):
                            cases, this argument does not need to be set
                            explicitly.
     :param logger: To enable logging set to ``True`` or pass a logger object to
-                   use. To disable logging set to ``False``.
+                   use. To disable logging set to ``False``. The default is
+                   ``False``.
     :param binary: ``True`` to support binary payloads, ``False`` to treat all
                    payloads as text. On Python 2, if this is set to ``True``,
                    ``unicode`` values are treated as text, and ``str`` and
@@ -106,26 +115,32 @@ class SocketIO(object):
                        first async mode that has all its dependencies installed
                        is then one that is chosen.
     :param ping_timeout: The time in seconds that the client waits for the
-                         server to respond before disconnecting.
+                         server to respond before disconnecting. The default is
+                         60 seconds.
     :param ping_interval: The interval in seconds at which the client pings
-                          the server.
+                          the server. The default is 25 seconds.
     :param max_http_buffer_size: The maximum size of a message when using the
-                                 polling transport.
-    :param allow_upgrades: Whether to allow transport upgrades or not.
+                                 polling transport. The default is 100,000,000
+                                 bytes.
+    :param allow_upgrades: Whether to allow transport upgrades or not. The
+                           default is ``True``.
     :param http_compression: Whether to compress packages when using the
-                             polling transport.
+                             polling transport. The default is ``True``.
     :param compression_threshold: Only compress messages when their byte size
-                                  is greater than this value.
+                                  is greater than this value. The default is
+                                  1024 bytes.
     :param cookie: Name of the HTTP cookie that contains the client session
                    id. If set to ``None``, a cookie is not sent to the client.
+                   The default is ``'io'``.
     :param cors_allowed_origins: List of origins that are allowed to connect
                                  to this server. All origins are allowed by
                                  default.
     :param cors_credentials: Whether credentials (cookies, authentication) are
-                             allowed in requests to this server.
+                             allowed in requests to this server. The default is
+                             ``True``.
     :param engineio_logger: To enable Engine.IO logging set to ``True`` or pass
                             a logger object to use. To disable logging set to
-                            ``False``.
+                            ``False``. The default is ``False``.
     """
 
     def __init__(self, app=None, **kwargs):
@@ -162,7 +177,7 @@ class SocketIO(object):
             channel = self.server_options.pop('channel', 'flask-socketio')
             write_only = app is None
             if url:
-                if url.startswith('redis://'):
+                if url.startswith(('redis://', "rediss://")):
                     queue_class = socketio.RedisManager
                 elif url.startswith('zmq'):
                     queue_class = socketio.ZmqManager
@@ -367,7 +382,7 @@ class SocketIO(object):
         used outside of a SocketIO event context, so it is appropriate to use
         when the server is the originator of an event.
 
-        :param message: The message to send, either a string or a JSON blob.
+        :param data: The message to send, either a string or a JSON blob.
         :param json: ``True`` if ``message`` is a JSON blob, ``False``
                      otherwise.
         :param namespace: The namespace under which the message is to be sent.
@@ -526,7 +541,7 @@ class SocketIO(object):
                     log=log, **kwargs)
             else:
                 self.wsgi_server = pywsgi.WSGIServer((host, port), app,
-                                                     log=log)
+                                                     log=log, **kwargs)
 
             if use_reloader:
                 # monkey patching is required by the reloader
@@ -583,9 +598,10 @@ class SocketIO(object):
         """
         return self.server.sleep(seconds)
 
-    def test_client(self, app, namespace=None):
+    def test_client(self, app, namespace=None, query_string=None, headers=None):
         """Return a simple SocketIO client that can be used for unit tests."""
-        return SocketIOTestClient(app, self, namespace)
+        return SocketIOTestClient(app, self, namespace=namespace,
+                                  query_string=query_string, headers=headers)
 
     def _handle_event(self, handler, message, namespace, sid, *args):
         if sid not in self.server.environ:
@@ -598,7 +614,7 @@ class SocketIO(object):
                 # created as a copy of the regular user session
                 if 'saved_session' not in self.server.environ[sid]:
                     self.server.environ[sid]['saved_session'] = \
-                        dict(flask.session)
+                        _ManagedSession(flask.session)
                 session_obj = self.server.environ[sid]['saved_session']
             else:
                 # let Flask handle the user session
@@ -693,6 +709,8 @@ def send(message, **kwargs):
     can only be called from a SocketIO event handler.
 
     :param message: The message to send, either a string or a JSON blob.
+    :param json: ``True`` if ``message`` is a JSON blob, ``False``
+                     otherwise.
     :param namespace: The namespace under which the message is to be sent.
                       Defaults to the namespace used by the originating event.
                       An empty string can be used to use the global namespace.
@@ -713,6 +731,7 @@ def send(message, **kwargs):
                          single addresee. It is recommended to always leave
                          this parameter with its default value of ``False``.
     """
+    json = kwargs.get('json', False)
     if 'namespace' in kwargs:
         namespace = kwargs['namespace']
     else:
@@ -726,7 +745,7 @@ def send(message, **kwargs):
     ignore_queue = kwargs.get('ignore_queue', False)
 
     socketio = flask.current_app.extensions['socketio']
-    return socketio.send(message, namespace=namespace, room=room,
+    return socketio.send(message, json=json, namespace=namespace, room=room,
                          include_self=include_self, callback=callback,
                          ignore_queue=ignore_queue)
 
